/*
	MediaCup B V 0.0-0 and V 0.1-0

	written 1.12.2000 by Michael Beigl
	based on MediaCup H 1.3 Code by Holger Krull
	TecO, University of Karlsruhe
	mediacup.teco.edu

	this software may be used under the MediaCup licence agreement
	(see mediacuplicence.txt or http://mediacup.teco.edu/tech/mediacuplicence.txt)

	Main File of MediaCup B Project
	other files needed: ds1621.c, 16f84.h
	build for CSS compiler (PCM)

	History
	-					1.12.2000	Michael Beigl

*/
#include <16f84.h>

#fuses xt,wdt,noprotect

#use delay(clock=1000000,RESTART_WDT)


// Here enter the IP adress of the MediaCup
#define IP0	92		// IP Adress byte 0
#define IP1	73
#define IP2	10
#define IP3	13


#define WDT_CYCLE WDT_36MS
	/*  WDT_18MS		WDT_36MS		WDT_72MS		WDT_144MS
		WDT_288MS		WDT_576MS		WDT_1152MS		WDT_2304MS
	*/
#define NO_OF_CYCLES    64	// 64 cycle with 36 ms leads to a transmission all 2 sec (64x32)
// at 1 MHz with 36 MS Cycle Drink cycle must be 10 (= 400ms):
#define DRINK_CYCLE 	12
#define TOLERANCE	25		// tolerance for walk state
#define PLAY_COUNT	12		// minimum number of changes that makes up the play state

#define irpin PIN_A2
#define ledpin PIN_A3

#define ball_switch1 1		// PIN_B0 on PIC 16F84
#define ball_switch2 2		// PIN_B1
#define ball_switch3 16		// PIN_B4


// these are needed for the ds1621c library
#define i2c_sda PIN_B3
#define i2c_scl PIN_B2

#use i2c(master,sda= i2c_sda, scl= i2c_scl , fast, RESTART_WDT)
#include <Ds1621.c>

// Some constants
#define temp_data	1
#define temp_data_b	2
#define state_b       16
#define desk 		1
#define drink	 	2
#define play	 	3
#define unknown 	4
#define sleepS		5
// #define walk		6

int suspended=FALSE;
char state;
unsigned int CRC8;			// Buffer for "incremental" CRC computation
unsigned int packet[16];		// Buffer for packet to send
unsigned int * end_of_packet;			// pointer to actual end of buffer in packet


typedef union z1 {
	unsigned long counter;
	struct zb {
		unsigned int counterl;
		unsigned int counterh;
	} bytes;
} type_suspend_counter;

int abs(signed int i)
{
   int r;

   r = (i < 0) ? -i : i;
   return(r);
}




bytesend( unsigned int value)
{
	*end_of_packet=value;
	end_of_packet++;
}

// 19200 Send Procedure; bit length is 52 us
#define interval 40
bytesend_w(unsigned int Buffer)
{

#use fast_io(A)

	output_high(irpin);
	output_low(irpin);
	delay_us(interval);

	if (!bit_test(Buffer,0)) output_high(irpin);
	output_low(irpin);
	delay_us(interval);

	if (!bit_test(Buffer,1)) output_high(irpin);
	output_low(irpin);
	delay_us(interval);

	if (!bit_test(Buffer,2)) output_high(irpin);
	output_low(irpin);
	delay_us(interval);

	if (!bit_test(Buffer,3)) output_high(irpin);
	output_low(irpin);
	delay_us(interval);

	if (!bit_test(Buffer,4)) output_high(irpin);
	output_low(irpin);
	delay_us(interval);

	if (!bit_test(Buffer,5)) output_high(irpin);
	output_low(irpin);
	delay_us(interval);

	if (!bit_test(Buffer,6)) output_high(irpin);
	output_low(irpin);
	delay_us(interval);

	if (!bit_test(Buffer,7)) output_high(irpin);
	output_low(irpin);
	delay_us(interval);

	delay_us(44);

}


// Sends data collected in global variable packet via infrared
// pointers are not set back !
void packetsend()
{
char * p;

	p=packet;

	while( p<end_of_packet )
	{
		bytesend_w( *p );
		p++;
	}
}


//CRC procedure uses global variable !
unsigned int crc_det( unsigned int value)
{
unsigned int i, f;

  for( i=0; i<8; ++i )
  {
  	f = (value ^ CRC8) & 1;
  	CRC8 >>= 1;
  	value >>= 1;
  	if (f) CRC8 ^= 0x8c;
  }
  return CRC8;
}


// 0 ,192, 219 are to be escaped, Escape character is 219
void bytesend_CRC_ESC(unsigned int value)
{
	crc_det( value );
	if (value == 0 || value == 219 || value == 192 )
	{
		bytesend(219);
		++value;
	}
	bytesend(value);
}


// Send IP at the packet begin, set CRC and buffer pointer back
// packetlen = len
void ID_send_wo_CRC(int len)
{

	end_of_packet= packet;					// send buffer to beginning
	CRC8 = 0;						// CRC check reset
	bytesend(0);				// sync
	bytesend_CRC_ESC(len);			// lengh without sync and CRC, including length
	bytesend_CRC_ESC(IP0);			// IP-Adress0
	bytesend_CRC_ESC(IP1);			// Adress
    bytesend_CRC_ESC(IP2);			// Adress
    bytesend_CRC_ESC(IP3);			// Adress

}


void ID_send()
{
	ID_send_wo_CRC(5);
	bytesend(CRC8);
}



#byte   portb  =  6

void d16bit_send( byte typ , signed long * value )
{
	bytesend_CRC_ESC( typ );
	bytesend_CRC_ESC( * (byte*)value );
	bytesend_CRC_ESC( * (byte*)(value+1) );
}

void d8bit_send( byte typ , signed int value )
{
	bytesend_CRC_ESC( typ );
	bytesend_CRC_ESC( value );
}


#byte   portb = 6
// read out button state, return value: button pressed = 1
char s_get()
{
char t;
	set_tris_b(0b11111111);			// select input
	t = portb & 0b00010011;			// mask out other input
	t ^= 0b11111111;			// invert makes button pressed = 1 (by default, buttons are not pressed leading to high=1
						// in the register. If the buttons are pressed (or the ball is moving in the ball switch)
						// a 0 can be read in the register portb. )
	return t;
}

#use standard_io(A)
void ledon(char time)
{
	output_low( ledpin );
	delay_ms( time );
	output_float( ledpin );
}



main()
{
char ball_switch,old_ball_switch,changes;
signed int temp;

int i=0;
unsigned int play_counter=0,n1counter_high=0,n2counter_high=0,n3counter_high=0;
unsigned int n1counter_low=0,n2counter_low=0,n3counter_low=0;

type_suspend_counter go_suspend;


#use standard_io(A)

	port_b_pullups(TRUE);
	output_low( irpin );

	init_temp();
 	temp_config(9);		// 9 = single meassurement 8 = continous meassurement in temperature sensor
	start_temp();

	end_of_packet = packet;

	suspended=FALSE;
	ledon( 200 );
	ledon( 200 );

	SETUP_COUNTERS(RTCC_INTERNAL,WDT_CYCLE);

	old_ball_switch=s_get();
	ID_send();

	while( TRUE )
	{
		sleep();

		if (++i > NO_OF_CYCLES )		// all 2 seconds (= NO_OF_CYCLES*WDT_CYCLE)  send the data !
		{

			if( suspended )		// Tasse should go to suspend mode
			{
				if( state != sleepS )
				{									// Tell other objects about that
					state = sleepS;

					ID_send_wo_CRC(5+2);
					d8bit_send( state_b, sleepS );
					bytesend(CRC8);
					bytesend(192);
					packetsend();				// Send the packet previously build

				}

			}
			else
			{
				start_temp();		// start measurement of temperature, max time to result: 1 sec.
				i=0;			// set back counter

				state = unknown;	// default state: unknown
				ball_switch=s_get();
				if( (play_counter==0)&&(n1counter_high==0)&&(n2counter_high==0)
					&&(n3counter_high==0) )
				{
					state = desk;
				}
				else
				{
					if (((n1counter_high>DRINK_CYCLE) || (n2counter_high>DRINK_CYCLE)
						||(n3counter_high>DRINK_CYCLE)) && (play_counter < 6))
					{ state = drink; }
					else
					{
						state = play;
					}

				}
			}

			temp = read_temp_b();

			//ID_send();
			ID_send_wo_CRC(5+2+2);
			d8bit_send( temp_data_b, temp );
			d8bit_send( state_b, state );
			bytesend(CRC8);
			bytesend(192);

			packetsend();		// send packet

			// set back counters
			play_counter=0;
			n1counter_low=0;
			n2counter_low=0;
			n3counter_low=0;
			n1counter_high=0;
			n2counter_high=0;
			n3counter_high=0;

		}
		else
		{	// retrieval of sensor data
			ball_switch=s_get();

			changes=ball_switch ^ old_ball_switch;
			if( changes  )
			{	// changes
				if (changes & ( ball_switch1+ball_switch2+ball_switch3) )  play_counter++;
				go_suspend.counter=0;
				suspended=FALSE;

			}
			else
			{
				if (ball_switch & ball_switch1) n1counter_high++;
				else n1counter_low++;
				if (ball_switch & ball_switch2) n2counter_high++;
				else n2counter_low++;
				if (ball_switch & ball_switch3) n3counter_high++;
				else n3counter_low++;

				if( go_suspend.bytes.counterh > 146 ) // high value to suspend is 146 = 20 minutes
				{
					suspended=TRUE;
				}
				else
				{
					++go_suspend.counter;
				}

				go_suspend++;
			}

		}
		old_ball_switch=ball_switch;
	}
}
